# -*- coding: iso_8859_1 -*-

import numpy as np
import math

def air_mass(alpha, delta, lat, long, JJ):
	"""
	---------------------
	Purpose
	Calculates air mass ratio for specific location, time and sky coordinates of an object 
	---------------------
	Inputs
	* alpha (string) = right ascension, string with format '13:34:12.45'
	* delta (string) = declination, string with format '-06:34:23.1'
	* lat (string) = observing geodetic latitude, string with format '+52:09:12.34' 
	* long (string) = observing longitude, positive East of Greenwich, string with format '+04:12:23.3'
	* JJ (float) = julian date for which air mass ratio must be calculated
	Allowed input formats for alpha are:
		'13:34:12.45'
		'13:34:12'	
	Allowed input formats for delta, lat, long are:
		'-06:34:23.1'
		'-06:34:23'
		'+06:34:23.1'
		'+06:34:23'
		'06:34:23.1'
		'06:34:23'
	---------------------
	Output (float) = air mass ratio
	---------------------
	"""

	#alpha, delta, lat, long in degrees
	#long is positive East of Greenwich
	
	#sideral time at Greenwich, 0h UT [in hours]
	T = (JJ-2415020.)/36525.
	theta0 = 6.6460656 + 2400.051262*T + 0.00002581*T*T
	
	#sideral time at Greenwich [in degrees]
	theta = np.mod(theta0 + 1.002737908 * (JJ-np.floor(JJ) + 0.5) * 24., 24.) * 15.
	
	#angle horaire [in degrees]
	H = np.mod(theta + long - alpha, 360.)
	
	#elevation [in degrees]
	k = math.pi/180.
	elevation = (1./k)*np.arcsin(np.sin(lat*k)*np.sin(delta*k)+np.cos(lat*k)*np.cos(delta*k)*np.cos(H*k))
	
	#air mass ratio
	dist = 90.-elevation
	air_mass = (1./np.cos(dist*k))*(1.-0.0012*(np.tan(dist*k)**2.))
	
	return air_mass

def hours_to_decdegrees(hours):
	"""
	---------------------
	Purpose
	Transforms string 'hours:minutes:seconds' to decimal number in degrees
	---------------------
	Inputs
	* hours (string) = string to be converted
	No minus sign is allowed in the input string. Allowed formats are:
		'13:34:12.45'
		'13:34:12'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = hours.split(":")
	return 15.*(np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.)
	
def degrees_to_decdegrees(degrees):
	"""
	---------------------
	Purpose
	Transforms string 'degrees:arcmin:arcsec' to decimal number in degrees
	---------------------
	Inputs
	* degrees (string) = string to be converted
	A plus or minus sign is allowed for the first character. Allowed formats are:
		'-06:34:23.1'
		'-06:34:23'
		'+06:34:23.1'
		'+06:34:23'
		'06:34:23.1'
		'06:34:23'
	---------------------
	Output (float) = result in degrees
	---------------------
	"""
	values = degrees.split(":")
	if values[0][0] == '-':
		result = -(np.abs(np.float(values[0]))+np.float(values[1])/60.+np.float(values[2])/3600.)
	else:
		result = np.float(values[0])+np.float(values[1])/60.+np.float(values[2])/3600.
	return result
