# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *

def add_sigma_clipping2(images, sigma, iterations):
	"""
	---------------------
	Purpose
	Combines a list of images with the sigma clipping method
	The method works pixel by pixel: for one pixel, it calculates the mean and std_deviation values on all images, then the pixel values deviating from the mean by sigma * std_deviation are rejected.
	The process is repeated iterations times, and the mean of the remaining pixel values is kept for the final image.
	The method is not recommended with a low number (<15) of images. For a higher number of images it is very efficient.
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* sigma (float) = the pixel values that deviate from the mean value by more than sigma * std_deviation are rejected
	* iterations (integer) = the process of calculating mean, std deviation and testing to reject pixel values is repeated iterations times
	---------------------
	Output (2D Numpy array) = combined image
	---------------------
	"""
	dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim_x*dim_y) 
	C=imag_out()

	sigma_dll = c_double(sigma)

	cdll.sigma_clipping.Sigma_clipping(dim,dim_x,dim_y,B,sigma_dll,iterations,byref(C))

	result = []
	result[:] = C[:]

	return np.reshape(result,(dim_x,dim_y))

def add_sigma_clipping(images, sigma, iterations, cutsize=[500,500]):
	"""
	---------------------
	Purpose
	Combines a list of images with the sigma clipping method
	This function must be used for very large images: it cuts the images into smaller subparts, and combines the subparts with astro.add_sigma_clipping2().
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* sigma (float) = the pixel values that deviate from the mean value by more than sigma * std_deviation are rejected
	* iterations (integer) = the process of calculating mean, std deviation and testing to reject pixel values is repeated iterations times
	* cutsize ([integer, integer]) = 2 elements list giving the size of the largest subimage to be sent to astro.add_sigma_clipping2()
	---------------------
	Output (2D Numpy array) = combined image
	---------------------
	"""
	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = dim_x/sx + 1
	ny = dim_y/sy + 1
	
	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_sigma_clipping2(images[:,i1:i2,j1:j2], sigma, iterations)

	return result

def add_mean(images):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the mean value of the pixels over the image list
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	---------------------
	Output (2D Numpy array) = average image
	---------------------
	"""
	return np.mean(images, axis=0)

def add_median2(images):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the median value of the pixels over the image list
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	---------------------
	Output (2D Numpy array) = combined median image
	---------------------
	"""
	dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------

	imag_out = c_double * (dim_x*dim_y) 
	C=imag_out()

	cdll.median.Median(dim,dim_x,dim_y,B,byref(C))

	result = []
	result[:] = C[:]
	return np.reshape(result,(dim_x,dim_y))

def add_median(images, cutsize=[1000,1000]):
	"""
	---------------------
	Purpose
	Combines a list of images by calculating the median value of the pixels over the image list
	This function must be used for very large images: it cuts the images into smaller subparts, and combines the subparts with astro.add_median2().
	---------------------
	Inputs
	* images (3D Numpy array) = input image data to be combined, as obtained by reading a list of image files with the function astro.get_imagedata().
	* cutsize ([integer, integer]) = 2 elements list giving the size of the largest subimage to be sent to astro.add_median2()
	---------------------
	Output (2D Numpy array) = combined median image
	---------------------
	"""

	sx, sy= cutsize
	dim, dim_x, dim_y = images.shape

	nx = dim_x/sx + 1
	ny = dim_y/sy + 1
	
	result = np.zeros(np.shape(images)[1:3])

	for i in range(nx):
		for j in range(ny):
			i1 = i*sx
			i2 = min((i+1)*sx,dim_x)
			j1 = j*sy
			j2 = min((j+1)*sy,dim_y)
			result[i1:i2,j1:j2] = add_median2(images[:,i1:i2,j1:j2])

	return result

