# -*- coding: iso_8859_1 -*-

import datetime, time
import numpy as np
import load
import aspylib.conf as conf


def create_headers(Config, Imagelist):
	"""
	---------------------
	Purpose
	Extracts FITS headers from a list of FITS images, and modify them according to some rules defined in the file conf.py.
	The result is a complete set of FITS headers for each image, that can be used for instance to save the image list after modifications (e.g. calibration, filtering, etc).
	---------------------
	Inputs
	* Config (integer) = defines which set of rules must be used to modify the FITS headers. The file conf.py allows to define several sets of rules, and specifies which headers must be removed, kept, or added.
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) for the ith image in the list. This list of image is used as a starting point to generate the FITS headers.
	---------------------
	Output (list) = has the form [dictionary_1, dictionary_2, ..., dictionary_n]. Each list element dictionary_i is a Python dictionary that contains all the headers for the ith FITS image.
	The dictionary elements have the form {'KEYWORD':[value,'comment']}.
	---------------------
	"""

	hdictconfig = conf.set_headers(Config)
	hdictlist = load.get_headers(Imagelist, text_display=False)		#list of header dictionaries from image list
	hdictlistnew = []												#new list of header dictionaries

	for hdict in hdictlist:
		hdictnew = hdictconfig
		for key in hdictnew.keys():
			if hdictnew[key]==[]:
				if key in hdict:
					hdictnew[key] = hdict[key]
				else:
					del hdictnew[key]

		hdictlistnew.append(hdictnew)

	return hdictlistnew
	

def julian_day(dateobs):
	"""
	---------------------
	Purpose
	Converts the value of a FITS header representing a date and provided in the format "YYYY-MM-DDThh:mm:ss.sss" (e.g. headers DATE, DATE-OBS or DATE-END), to a Julian date
	---------------------
	Inputs
	* dateobs (string) = the FITS header value to be converted to a Julian date, with a format equivalent to "2011-03-21T23:12:34.323"
	Digits after the "." separator, representing a fraction of a second, are not mandatory. If they are present, they will be considered
	---------------------
	Output (float) = Julian date
	---------------------
	"""

	time_format = "%Y-%m-%dT%H:%M:%S"
	result = []
	for dd in dateobs:
		dd2 = dd.split('.')[0]
		dd3 = datetime.datetime.fromtimestamp(time.mktime(time.strptime(dd2, time_format)))

		year = dd3.year
		month = dd3.month
		
		day = dd3.day + dd3.hour/24. + dd3.minute/(24.*60.) + dd3.second/(24.*3600.)
		
		if len(dd.split('.'))>1:
			if len(dd.split('.')[1])>0:
				day = day + float('.'+dd.split('.')[1])/(24.*3600.)

		if month<=2:
			month = month+12
			year = year-1
		
		a = np.trunc(year/100.)
		b = 2. - a + np.trunc(a/4.)
			
		result.append(np.trunc(365.25*year)+np.trunc(30.6001*(month+1.))+day+1720994.5+b)
	return result