# -*- coding: iso_8859_1 -*-

import pyfits
import numpy as np
import os
import aspylib.conf as conf

def get_imagedata(Imagelist, text_display=True):
	"""
	---------------------
	Purpose
	Loads the image data of a list of FITS images.
	Returns all the data in the form of a single array.
	---------------------
	Inputs
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. The dimensions of the Numpy array are (Image number, Vertical direction, Horizontal direction).
	---------------------
	"""

	if text_display:
		print
		print "--- loading FITS image ---"
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
			
		try:
			bzero = hdulist[0].header['bzero']
		except KeyError:
			bzero = 0.
		conf.update_saturation(32767. + bzero)

		if i==0:
			imagedata = np.array(hdulist[0].data, dtype=np.float32)
			if len(Imagelist)==1:
				imagedata = np.tile(imagedata,(1,1))
			else:
				imagedata = np.tile(imagedata,(len(Imagelist),1,1))
		else:
			imagedata[i,:,:] = np.array(hdulist[0].data, dtype=np.float32)

	hdulist.close()
	return imagedata

	
def get_someimagedata(Imagelist, area, text_display=True):
	"""
	---------------------
	Purpose
	This function is the same as astro.get_imagedata(), except that only a subpart of the images is loaded.
	For large images, if only a small part of the images is needed this function is much faster than astro.get_imagedata().
	---------------------
	Inputs
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image file.
	* area (list) = has the form [x1, x2, y1, y2]. The values x1, x2, y1, y2 are pixel coordinates. The part of the images which is read out is [x1:x2,y1:y2] with x being the vertical coordinate, y the horizontal coordinate.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (2D or 3D Numpy array) = image data, 2D Numpy array for a single image, or 3D Numpy array for a list of images. The dimensions of the Numpy array are (Image number, Vertical direction, Horizontal direction).
	---------------------
	"""

	if text_display:
		print
		print "--- loading part of FITS image ---"
		print "loaded pixels:[",area[0],":",area[1],",",area[2],":",area[3],"]"
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
			
		try:
			bzero = hdulist[0].header['bzero']
		except KeyError:
			bzero = 0.
		conf.update_saturation(32767. + bzero)
		
		if i==0:
			imagedata = hdulist[0].section[area[0]:area[1],area[2]:area[3]]
			if len(Imagelist)==1:
				imagedata = np.tile(imagedata,(1,1))
			else:
				imagedata = np.tile(imagedata,(len(Imagelist),1,1))
		else:
			imagedata[i,:,:] = hdulist[0].section[area[0]:area[1],area[2]:area[3]]

	hdulist.close()
		
	return np.array(imagedata, dtype=np.float32)	

	
def get_headers(Imagelist, text_display=True):
	"""
	---------------------
	Purpose
	This function reads all the FITS headers of a list of FITS images.
	The result is a list of Python dictionaries. Each dictionary corresponds to a single image, and contains all its headers.
	---------------------
	Inputs
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image file.
	* text_display (boolean) = optional argument. If equal to True (default), some information is written in the console window. If equal to False nothing is written.
	---------------------
	Output (list) = has the form [dictionary_1, dictionary_2, ..., dictionary_n]. Each list element dictionary_i is a Python dictionary that contains all the headers for the ith FITS image.
	The dictionary elements have the form {'KEYWORD':[value,'comment']}.
	---------------------
	"""
	
	if text_display:
		print 
		print "--- loading FITS headers ---"
	listheaders = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		if text_display:
			print sImage.split('\\')[-1]
		hdulist = pyfits.open(sImage)
		
		cardlist = hdulist[0].header.ascardlist()
		headerdict = {}
		for x in cardlist:
			headerdict[x.key] = [x.value, x.comment]

		if i==0:
			listheaders = [headerdict]
		else:
			listheaders.append(headerdict)

	hdulist.close()
	return listheaders

	
def get_headervalues(Imagelist, Header):
	"""
	---------------------
	Purpose
	This function reads all the values associated to a FITS header keyword in a list of FITS images.
	---------------------
	Inputs
	* Imagelist (list) = has the form [string_1, string_2, ..., string_n]. Each list element string_i is the complete filename (including path) of one FITS image file.
	* Header (string) = FITS header keyword that must be read out. It can be for instance: "NAXIS", "DATE-OBS", "OBJECT"...
	---------------------
	Output (list) = list with all the header values (one for each image), with the form [value1, value2, ..., valuen].
	---------------------
	"""
	
	headervalues = []
	
	#loading images
	for i, sImage in enumerate(Imagelist):
	
		try:
			hdulist = pyfits.open(sImage)
			headervalues.append(hdulist[0].header[Header])
		except KeyError:
			headervalues.append('')

	hdulist.close()
		
	return headervalues


def check_extension(file_name):
	"""
	---------------------
	Purpose
	This function checks the extension of a file name. It returns True if the extension is ".fit" or ".fits".
	---------------------
	Inputs
	* file_name (string) = filename (path can be included or not).
	---------------------
	Output (boolean) = True if the file has extension ".fit" or ".fits". False otherwise.
	---------------------
	"""
	ext = os.path.splitext(file_name)[1]
	if (ext==".fit") or (ext==".fits"):
		answer = True
	else:
		answer = False
	return answer