# -*- coding: iso_8859_1 -*-

import numpy as np
import process, process2
from ctypes import *


def get_median(images):
	"""
	---------------------
	Purpose
	Calculates the median values of all pixels for each image in a list of images
	The algorithm to calculate the median values is based on the Wirth method. A C implementation from Devillard (from http://ndevilla.free.fr/median/median/index.html) is used, after correction of a typo.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	---------------------
	Output (list) = Python list containing the median values of each image (float numbers).
	---------------------
	"""
	
	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------

	imag_out = c_double * (dim) 
	C=imag_out()

	cdll.median.Median2(dim,dim_x,dim_y,B,byref(C))

	result = []
	result[:] = C[:]
	return result
	

def mean_psf(image, detection_area, nb_stars, excluded=[], included=[]):
	"""
	---------------------
	Purpose
	Calculates the average parameters of the PSFs of the brightest stars on one image.
	The fits are performed with a 2D elliptical gaussian PSF.
	---------------------
	Inputs
	* image (2D Numpy array) = array that contains the data of one image (2D case) and for which the average PSF parameters must be computed.
	* detection_area (integer, must be >0) = defines the +/- size of the square window used for star fitting (the square window has a size 2*detection_area+1).
	The window size must be large enough to include the stars, but small enough not to include other stars which may perturbate the fit.
	* nb_stars (integer, must be >1) = number of stars requested. The function is using a number of stars between 0.5*nb_stars and 1.5*nb_stars.
	* excluded (list) = list of 4 elements lists, with the form [[x11,x12,y11,y12], ..., [xn1,xn2,yn1,yn2]]. Each element is a small part of the image that will not be used for star detection.
	This optional parameter allows to reject problematic areas in the image (e.g. saturated star(s), artefact(s), moving object(s)..)
	* included (list) = list of 4 integers, with the form [x1,x2,y1,y2]. It defines a subpart of the image where all the stars will be detected.
	---------------------
	Output (list) = list with 3 elements, in the form [fhwm_small, fwhm_large, angle]. The list elements are respectively:
	- fwhm_small is the average smallest full width half maximum of the elliptical gaussian PSF in pixels
	- fwhm_large is the average largest full width half maximum of the elliptical gaussian PSF in pixels
	- angle is the average angular direction of the largest fwhm, measured clockwise starting from the vertical direction and expressed in degrees. The average direction of the smallest fwhm is obtained by adding 90 deg to angle.
	---------------------
	"""
	
	#fitting stars
	stars = process2.find_stars(image, nb_stars, excluded=excluded, included=included, margin=detection_area)
	stars = process2.fit_stars(image, stars, detection_area, full_info=True)
	
	#picking information
	fwhm_1 = []
	fwhm_2 = []
	cosine = []
	sine = []

	for xy in stars:
		#print xy
		fwhm_1.append(xy[5])
		fwhm_2.append(xy[6])
		cosine.append(np.cos(xy[7]))
		sine.append(np.sin(xy[7]))
		
	return [np.mean(fwhm_1), np.mean(fwhm_2), np.arctan2(np.mean(sine),np.mean(cosine))]
