# -*- coding: iso_8859_1 -*-

import numpy as np
from ctypes import *


def translate_Bspline(images, shift, spline_degree):
	"""
	---------------------
	Purpose
	Translates an image (or list of images) by non-integer shifts in both X and Y directions.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the translated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* shift ([float, float]) = 2 elements list that contains respectively the shifts in X (vertical) and Y (horizontal) directions.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""
	
	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape
		
	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.Bspline.Translate(dim,dim_x,dim_y,B,byref(dxy),spline_degree,byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def rotate_Bspline(images, xy_axis, angle, spline_degree):
	"""
	---------------------
	Purpose
	Rotates an image (or list of images) by an arbitrary angle, with an arbitrary (non-integer, possibly outside the image) position of the rotation axis.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the rotated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_axis ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the rotation axis.
	* angle (float) = rotation angle in degrees, measured clockwise.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.Bspline.Rotate(dim,dim_x,dim_y,B,byref(xy_dll),angle_dll,spline_degree,byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def scale_Bspline(images, xy_center, scaling, spline_degree):
	"""
	---------------------
	Purpose
	Scales an image (or list of images) by an arbitrary factor (larger or smaller than 1), with an arbitrary (non-integer, possibly outside the image) position of the stationary pixel.
	B-Spline interpolation with arbitrary order (from 2 to 9) is used to calculate the scaled pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_center ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the stationary pixel.
	* scaling (float, must be >0) = scaling factor.
	* spline_degree (integer) = the degree of the B-Spline that is used for interpolation : it can be 2, 3, 4, 5, 6, 7, 8 or 9.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]

	scaling_dll = c_double(scaling)
	
	cdll.Bspline.Scale(dim,dim_x,dim_y,B,byref(xy_dll),scaling_dll,spline_degree,byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))		
	return result

def translate_bilinear(images, shift):
	"""
	---------------------
	Purpose
	Translates an image (or list of images) by non-integer shifts in both X and Y directions.
	Bilinear interpolation is used to calculate the translated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* shift ([float, float]) = 2 elements list that contains respectively the shifts in X (vertical) and Y (horizontal) directions.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""
	
	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	shift_array = c_double * 2
	dxy = shift_array()
	dxy[:] = shift[:]
	
	cdll.bilinear.Translate(dim,dim_x,dim_y,B,byref(dxy),byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def rotate_bilinear(images, xy_axis, angle):
	"""
	---------------------
	Purpose
	Rotates an image (or list of images) by an arbitrary angle, with an arbitrary (non-integer, possibly outside the image) position of the rotation axis.
	Bilinear interpolation is used to calculate the rotated pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_axis ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the rotation axis.
	* angle (float) = rotation angle in degrees, measured clockwise.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""

	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	axis_dll = c_double * 2
	xy_dll = axis_dll()
	xy_dll[:] = xy_axis[:]
	
	angle_dll = c_double(angle)
	
	cdll.bilinear.Rotate(dim,dim_x,dim_y,B,byref(xy_dll),angle_dll,byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

def scale_bilinear(images, xy_center, scaling):
	"""
	---------------------
	Purpose
	Scales an image (or list of images) by an arbitrary factor (larger or smaller than 1), with an arbitrary (non-integer, possibly outside the image) position of the stationary pixel.
	Bilinear interpolation is used to calculate the scaled pixels.
	---------------------
	Inputs
	* images (2D or 3D Numpy array) = array that contains the data of one image (2D case) or the data of a list of images (3D case).
	* xy_center ([float, float]) = 2 elements list that contains respectively the X (vertical) and Y (horizontal) positions of the stationary pixel.
	* scaling (float, must be >0) = scaling factor.
	---------------------
	Output (2D or 3D Numpy array) = transformed image data.
	---------------------
	"""
	
	if len(images.shape)==2:
		dim = 1
		dim_x, dim_y = images.shape
	else:
		dim, dim_x, dim_y = images.shape

	#-------------------------------------
	A = images.flatten()
	c_double_p = POINTER(c_double)
	A = A.astype(np.double)
	B = A.ctypes.data_as(POINTER(c_double_p))
	#-------------------------------------
	
	imag_out = c_double * (dim*dim_x*dim_y) 
	C=imag_out()

	center_dll = c_double * 2
	xy_dll = center_dll()
	xy_dll[:] = xy_center[:]
	
	scaling_dll = c_double(scaling)
	
	cdll.bilinear.Scale(dim,dim_x,dim_y,B,byref(xy_dll),scaling_dll,byref(C))

	result = []
	result[:] = C[:]

	if len(images.shape)==2:
		result = np.reshape(result,(dim_x,dim_y))
	else:
		result = np.reshape(result,(dim,dim_x,dim_y))
	return result

