# -*- coding: iso_8859_1 -*-

from aspylib import astro, conf
import astrometry2, astrometry3, astrometry5
import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize

def find_astrometry_for_image(image, proj, coef = [[],[]], distcrit = 5, nbdetstars = [60,200], nbcatstars = [100,1000], text_display = True, plotting=True): 

	#purpose is to find astrometric coefficients, including distorsion, to transform image pixel coordinates into stereographic projected coordinates

	#---------------------------------------------------------------------------------------------------------
	#A first estimate of the mapping coefficients (e.g. as obtained with a previous image of the same field) can be provided to speed up calculations:
	#	coef = [coef0, coef1]
	#	coef0 = [scaling, phi, dx, dy]
	#	coef1 = [a00, a10, a01, a20, a11, a02, a30, a21, a12, a03, b00, b10, b01, b20, b11, b02, b30, b21, b12, b03]
	#with
	#	[x2,y2] = scaling * [[cos(phi),-sin(phi)],[sin(phi),cos(phi)]]*[x1,y1] + [dx,dy]
	#	[x2,y2] = [a00 + a10*x1 + a01*y1 + a20*x1^2 + a11*x1*y1 + a02*y1^2 + a30*x1^3 + a21*x1^2*y1 + a12*x1*y1^2 + a03*y1^3,
	#			 b00 + b10*x1 + b01*y1 + b20*x1^2 + b11*x1*y1 + b02*y1^2 + b30*x1^3 + b21*x1^2*y1 + b12*x1*y1^2 + b03*y1^3]
	#---------------------------------------------------------------------------------------------------------

	#*********************************************************************************************************
	#		(1) preparing inputs
	#*********************************************************************************************************

	#--- catalog subsets ---
	proj1 = proj[:nbcatstars[0],:]
	proj2 = proj[:nbcatstars[1],:]

	#--- finding stars on image ---
	stars = astro.find_stars(image, max(nbdetstars), saturation = [True,conf.saturation], text_display = text_display)
	stars = astro.centroid_stars(image, stars, 2)
	stars, indices = astro.remove_duplicates(stars)
	stars1 = stars[:nbdetstars[0],:]
	stars2 = stars[:nbdetstars[1],:]

	#*********************************************************************************************************
	#		(2) first astrometric solution
	#*********************************************************************************************************

	#--- approximate solution ---
	if len(coef[0])==0:
		#scaling + rotation + translation that transform the pixel coordinates into stereographic coordinates
		scaling, phi, dx, dy = astrometry2.approximate_astrometry(stars1, proj1, nb_iterations=2, resolution=1024)
		newcoef = [[scaling, phi, dx, dy],[]]
		p0 = [dx, scaling*np.cos(phi), -scaling*np.sin(phi), 0., 0., 0., 0., 0., 0., 0., dy, scaling*np.sin(phi), scaling*np.cos(phi), 0., 0., 0., 0., 0., 0., 0.]
	else:
		#here approximate solution is known so we correct only for a small shift
		#----------------------------------------------------------------------------------------
		#	ASSUMPTION : accurate mapping (20 coefts) is used to estimate the shift corrections
		#	then it is only needed to have a translation between 2 consecutive accurate mappings
		#	To use the approximate mapping (4 coefts) to derive the shift corrections, then it is needed
		#	to have only a translation between approximate mappings of first and last images in the night --> the match can be progressively lost
		#----------------------------------------------------------------------------------------
		#temp1 = astrometry5.transform_stars(coef[0], stars1)			#this one is not used	
		temp1 = astrometry5.distorsion_law(coef[1], stars1)
		#----------------------------------------------------------------------------------------
		dx2, dy2 = astrometry2.find_translation(temp1, proj1, resolution=512, text_display=False, plotting=False)
		scaling, phi, dx, dy = coef[0]
		newcoef = [[scaling, phi, dx+dx2, dy+dy2],[]]
		p0 = list([coef[1][0]+dx2]) + list(coef[1][1:10]) + list([coef[1][10]+dy2]) + list(coef[1][11:])
	
	#*********************************************************************************************************
	#		(3) display results for checking
	#*********************************************************************************************************

	if text_display or plotting:
		#--- star transformation ---
		if len(coef[0])==0:
			trans1 = astrometry5.transform_stars(newcoef[0], stars1)
		else:
			trans1 = astrometry5.distorsion_law(p0, stars1)
		#--- calculate matching pairs ---
		x,y = astrometry3.matching_pairs(trans1,proj1,distcrit*newcoef[0][0])

	if text_display:
		print
		print "--- searching approximate astrometry ---"
		print "nb of stars used (detected / from catalog):",len(stars1),"/",len(proj1)
		if len(coef[0])==0:
			print "approximate astrometric coefficients:", scaling, phi, dx, dy
		else:
			print "approximate shift correction (on proj. plane):", dx2, dy2
			print "approximate shift correction (on image):", dx2/newcoef[0][0], dy2/newcoef[0][0]
		print "nb of matched pairs:" ,len(x)
	
	if plotting:
		#--- projects back catalog on image ---
		if len(coef[0])==0:
			invcoef = astrometry5.invert_simple_coefficients(newcoef[0])
			back1 = astrometry5.transform_stars(invcoef, proj1[:,:])
		else:
			back1 = astrometry5.inverse_distorsion_law(p0, proj1)
		
		#--- plots detected stars ---
		fig, ax = astro.display(image)
		for i in range(len(back1)):
			if i in y:
				ax.scatter(back1[i][1],back1[i][0],marker='+',color='g')		#scatter is using first argument as horizontal axis !
			else:
				ax.scatter(back1[i][1],back1[i][0],marker='+',color='r')
		plt.sca(ax)
		plt.title("Best solution for approximate astrometry") 
		plt.xlim([0,np.shape(image)[1]])
		plt.ylim([0,np.shape(image)[0]])
		plt.draw()

	#*********************************************************************************************************
	#		(4) accurate astrometry
	#*********************************************************************************************************

	#--- star transformation ---
	if len(coef[0])==0:
		trans2 = astrometry5.transform_stars(newcoef[0], stars2)
	else:
		trans2 = astrometry5.distorsion_law(p0, stars2)
	#--- calculate matching pairs ---
	x,y = astrometry3.matching_pairs(trans2,proj2,distcrit*newcoef[0][0])

	#--- accurate astrometry ---
	coordinates = np.vstack([stars2[x,0], stars2[x,1], proj2[y,0], proj2[y,1]]).transpose()
	errfunc = lambda p, xy: (astrometry5.distorsion_law(p, xy[:,:2]) - xy[:,2:]).flatten() # Distance to the target function
	p1, success = optimize.leastsq(errfunc, p0, args=(coordinates))
	newcoef[1] = p1
	
	#*********************************************************************************************************
	#		(5) display results for checking
	#*********************************************************************************************************

	if text_display:
		print
		print "--- searching accurate astrometry ---"
		print "nb of stars used (detected / from catalog):",len(stars2),"/",len(proj2)
		print "nb of matched pairs used for accurate astrometry:" ,len(x)
		print "astrometric coefficients:", p1

	if plotting:
		back2 = astrometry5.inverse_distorsion_law(p1,proj2[y[:],:])
		
		#--- plots detected stars ---
		fig, ax = astro.display(image)
		for i in range(len(back2)):
			ax.scatter(back2[i][1],back2[i][0],marker='+',color='c')		#scatter is using first argument as horizontal axis !
		plt.sca(ax)
		plt.title("Best solution for accurate astrometry") 
		plt.draw()

	return newcoef
