# -*- coding: iso_8859_1 -*-

import urllib, warnings
import numpy as np
from vo.table import parse_single_table

def send_query_skybot(date, position, conesize, obscode='500', outputlevel=1, webrequest = True, filename = 'skybot.xml'):
	"""
	---------------------
	Purpose
	Sends a query to SkyBoT and retrieve a list of solar system objects with astrometric (alpha, delta, etc) and photometric (magnitudes, etc) information
	The syntax to send a SkyBoT query is described there: http://vo.imcce.fr/webservices/skybot/?conesearch
	---------------------
	Inputs
	* date (float) = Julian date at which information is requested
	* position ([float,float]) = alpha/delta position (in degrees) giving the center of the sky area for which the request is formulated
	* conesize (None, float or [float,float]) = size in arcmin, corresponds to the radius (float) or alpha/delta box rectangular size ([float,float]) defining the sky area for which the request is formulated
	If equal to None (default), a radius of 5 arcmin is used
	* obscode (string) = optional parameter, default is '500'. Corresponds to the IAU observatory code for which information is requested. '500' corresponds to Earth center (geocentric positions).
	* outputlevel (integer) = optional parameter, default is 1.
	Defines the level of detail requested (see the SkyBoT webpage for more information).
	- 1 corresponds to 'object'
	- 2 corresponds to 'basic'
	- 3 corresponds to 'obs'
	- 4 corresponds to 'all'
	* webrequest (boolean) = optional parameter, default is True. If True, a web request is sent to Vizier. If False, an existing file previously obtained from SkyBoT and stored on the computer is used.
	* filename (string) = optional parameter, default is 'skybot.xml'.
	Specifies the xml file in which the data from the SkyBoT request is stored and read out by Python.
	It can be a full filename with path information, or a simple filename in which case the active directory is used.
	---------------------
	Output (Numpy record array) = array that contains all the data. The columns get their names from both the ID and name attributes of the FIELD elements in the VOTABLE file.
	See online help: http://stsdas.stsci.edu/astrolib/vo/html/intro_table.html#using-vo-table
	---------------------
	"""

	#------ (1) Preparing url for SkyBoT ----------------------

	dat = "-ep=" + str(date)
	pos = "&-ra=" + str(position[0]) + "&-dec=" + str(position[1])
	if conesize==None:
		cone = "&-rm=05"
	elif isinstance(conesize,list):
		cone = "&-bm=" + str(int(conesize[0])) + "x" + str(int(conesize[1]))
	else:
		cone = "&-rm=" + str(int(conesize))
	if outputlevel ==1:
		out = "&-output=object"
	elif outputlevel ==2:
		out = "&-output=basic"
	elif outputlevel ==3:
		out = "&-output=obs"
	else:
		out = "&-output=all"
	loc = "&-loc=" + obscode

	url = "http://vo.imcce.fr/webservices/skybot/skybotconesearch_query.php?" + dat + pos + cone + "&-mime=votable" + out + loc + "&-from=AsPyLib"
	print url

	#------ (2) Sending query to SkyBoT ----------------------

	if webrequest:
		print
		print "--- retrieving data from SkyBoT ---"
			
		fp = urllib.urlopen(url)
		op = open(filename, "wb")

		while 1:
			s = fp.read(8192)
			if not s:
				break
			op.write(s)

		fp.close()
		op.close()
	else:
		print
		print "--- reading existing .xml catalog ---"
		
	#------ (3) Converting votable to Numpy array -----------------

	warnings.simplefilter("ignore")
	votable = parse_single_table(filename, pedantic=False)
	warnings.resetwarnings() 
	data = votable.array
	
	return data
