# -*- coding: iso_8859_1 -*-

import numpy as np

def load_magfile(filename):
	"""
	---------------------
	Purpose
	Loads the magnitude data contained in one file, which corresponds to the measured lightcurve of one target (star or asteroid).
	Returns all the data in one Numpy array with the form [[d1, m1, mA1, mB1,...], [d2, m2, mA2, mB2,...], ...].
	- d1, d2, ..., dN are the Julian dates of the observations (start of exposure),
	- m1, m2, ..., mN are the measured magnitudes of the target (asteroid or star),
	- mA1, mA2, ..., mAN are the measured magnitudes for comparison star A,
	- mB1, mB2, ..., mBN are the measured magnitudes for comparison star B, etc.
	The function also returns the exposure time of the images (provided in the header of the file), as required to shift the Julian dates to mid-exposure so that they are accurate.
	---------------------
	Inputs
	* filename (string) = gives the complete filename (including path) of a file that contains the lightcurve measurements for one object (star or asteroid).
	The file format is the following:
	- the file contains a header part, followed by a data part
	- each line in the header part has its first character being a capital letter (A-Z), tabulation (\\t), end of line (\\n) or semicolon (;)
	- each line in the data part starts with a numeric character (0-9)
	- the data consists of n columns separated by tabulations
	- first column contains the Julian dates
	- second column contains the measured magnitude for the target
	- all other columns (at least one) correspond to the measured magnitudes for the comparison stars.
	The exposure time is given in the header of each lightcurve file, in seconds, after the following characters: "POS -1 ".
	For instance for a 30 sec exposure the file header contains the line "POS -1 30".
	---------------------
	Remarks
	- the number of comparison stars is arbitrary, but must be >=1.
	- the measured magnitudes are the ones directly obtained by aperture photometry on the calibrated images.
	- one file can contain several nights of observation, but must correspond to a single set of comparison stars
	---------------------
	Outputs = data, offset
	* data (Numpy array) = as described above.
	* offset (float) = exposure of each image, in seconds. This information is required to shift the Julian dates at mid-exposure.
	---------------------
	"""

	#display file name
	file_path = filename.split('\\')
	file_core = file_path[len(file_path)-1]
	print "... loading %s ..." % file_core

	#temporal offset for date
	offset = 0.
	
	#open file and reads data
	f=open(filename, 'r')
	try:
		data = []
		flag = 0
		for line in f:
			if (1-flag):
				if line[0:6]=='POS -1':
					offset = float(line.split()[2])/2.
				if line[0] not in ['A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z','\t','\n',';']:
					flag = 1
			if flag:
				datalist = line.split()
			
				#stores data in array
				x = []
				for dd in datalist:
					x.append(float(dd))
				data.append(x)
	finally:
		f.close()

	return np.array(data), offset

def load_file(filename):
	"""
	---------------------
	Purpose
	Generic function to load numeric data contained in a text file.
	The data must be arranged in columns, tab-separated.
	---------------------
	Inputs
	* filename (string) = gives the complete filename (including path) of the text file that contains the data.
	No header must be present in the file, only data.
	---------------------
	Output (Numpy array) = array with the same number of lines and columns as the data in the original file.
	---------------------
	"""

	#display file name
	file_path = filename.split('\\')
	file_core = file_path[len(file_path)-1]
	print "... loading %s ..." % file_core
	
	#open file and reads data
	f=open(filename, 'r')
	try:
		data = []
		for line in f:
			if line[0] not in ['\t','\n']:
				datalist = line.split()
			
				#stores data in array
				x = []
				for dd in datalist:
					x.append(float(dd))
				data.append(x)
	finally:
		f.close()
		
	return np.array(data)

def calc_noise(mag):
	"""
	---------------------
	Purpose
	This function provides an estimate of the high frequency magnitude noise in a lightcurve.
	The high-frequency noise or "local noise" is a better quantification of the noisy aspect of the lightcurve than the global rms noise, because it is still meaningful in the presence of slow, large amplitude variations (i.e. for variable stars, asteroids).
	First a smoothed lightcurve is calculated by filtering with a moving boxcar.
	The high-frequency noise is the rms noise of the lightcurve minus the smoothed version of it, after multiplication with the appropriate scaling factor.
	---------------------
	Inputs
	* mag (list or Numpy array) = list or Numpy array containing the magnitude values in the lightcurve.
	---------------------
	Output (float) = high-frequency magnitude noise.
	---------------------
	"""

	#this function estimates the noise of a magnitude curve. The principle is to smooth the curve with a moving window, and the difference between the 
	#initial curve and the smoothed curve is proportional to the noise
	#(other approaches are possible)
	if len(mag)<6:
		result = 'none'
	else:
		x = np.array(mag,dtype=np.float)
		y = x[2:-2] - (x[0:-4]+x[1:-3]+x[2:-2]+x[3:-1]+x[4:])/5.
		result = np.std(y,ddof=0) / np.sqrt(1.-1./5.)
		
	return result