# -*- coding: iso_8859_1 -*-

import cdr1
import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate

def load_magdata(Filelist):
	"""
	---------------------
	Purpose
	Loads a list of lightcurve files, and returns a list of Numpy arrays:
	- one array corresponds to one lightcurve file,
	- the Numpy arrays have 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	---------------------
	Definitions
	- DIFFERENTIAL MAGNITUDES are not true absolute magnitudes (the difference being an unknown offset),
	- MEASURED MAGNITUDES are the magnitude values obtained by aperture photometry on each images,
	- diff. magnitudes are obtained as the difference between meas. magnitudes for the target and the average meas. magnitudes of all comparison stars.
	---------------------
	Inputs
	* Filelist (list) = has the form [string_1, string_2, ..., string_n].
	Each string gives the complete filename (including path) of a file that contains the lightcurve measurements.
	The file format is the following:
	- the file contains a header part, followed by a data part
	- each line in the header part has its first character being a capital letter (A-Z), tabulation (\\t), end of line (\\n) or semicolon (;)
	- each line in the data part starts with a numeric character (0-9)
	- the data consists of n columns separated by tabulations
	- first column contains the Julian dates
	- second column contains the measured magnitude for the target
	- all other columns (at least one) correspond to the measured magnitudes for the comparison stars.
	---------------------
	Remarks
	- one file can contain several nights of observation, but must correspond to a single set of comparison stars
	- the dates in the input files are assumed to be at start of exposure. The function load_magdata() returns dates shifted at mid-exposure.
	The exposure time is given in the header of each lightcurve file, in seconds, after the following characters: "POS -1 ".
	For instance for a 30 sec exposure the file header contains the line "POS -1 30".
	---------------------
	Output (list) = list of 3 columns Numpy arrays, as described above.
	---------------------
	"""

	print
	print "------ loading measured data ------" 

	alldata = []

	for ff in Filelist:
		data, offset = cdr1.load_magfile(ff)
		
		#checking reference stars
		#plt.figure()
		#for i in range(np.shape(data)[1]-2):
		#	refstar = data[:,i+2]-np.mean(data[:,2:],axis=1) 
		#	plt.plot(refstar-np.mean(refstar)+0.02*np.double(i),label=str(i+1))
		#plt.legend()
		#plt.grid(True)
		#plt.show()

		#shifting date to mid-exposure
		data[:,0] = data[:,0] + offset/86400.

		#calculating differential magnitude
		data[:,1] = data[:,1] - np.mean(data[:,2:],axis=1)
		data = data[:,:2]
		#setting average magnitude to 0
		data[:,1] = data[:,1] - np.mean(data[:,1])
		
		alldata.append(data)

	print
	print "------ evaluating noise in measurements ------" 

	for i,dd in enumerate(alldata):
		noise = cdr1.calc_noise(dd[:,1])
		noise = np.tile(dd[:,0]*0.+noise,(1,1))
		alldata[i] = np.concatenate((dd.transpose(),noise)).transpose()

	return alldata

def plot_magdata(alldata, title):
	"""
	---------------------
	Purpose
	Plots the differential magnitude data, as returned by the function load_magdata(), as a function of Julian date.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays containing measured lightcurve data:
	- each array has 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	* title (string) = gives the title of the plot.
	---------------------
	Output = no output.
	---------------------
	"""

	plt.figure()
	for dd in alldata:
		plt.plot(dd[:,0],dd[:,1]-np.mean(dd[:,1]))
	plt.title(title)
	plt.grid()
	plt.show()

	
def time_correction(alldata, target, argument):
	"""
	---------------------
	Purpose
	Performs time corrections to a list of measured lightcurves.
	The corrections are as follows:
	- for asteroids the time delay due to light propagation from asteroid to Earth is subtracted from the Julian dates of the observations
	- for stars the Julian dates are converted to Heliocentric Julian dates
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays containing measured lightcurve data:
	- each array has 3 columns
	- the first column contains Julian dates of the observations,
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value.
	* target (integer) = 1 for asteroid, 2 for star
	* argument (string or [float,float])
	- if target = 1 (asteroid) 
	Then argument is a string giving the complete filename (including path) of a file that contains the distance Earth-asteroid as a function of time.
	--> the file starts immediately with numeric data. The data is provided in the form of 2 columns separated with a tabulation.
	--> first column contains Julian dates
	--> second column contains the Earth-asteroid distances, in astronomical units (AU)
	- if target = 2 (star)
	Then argument is a list [float, float], the two float numbers being respectively the right ascension (deg) and declination (deg) of the star target.
	Using these coordinates, the Julian dates are converted to Heliocentric Julian dates.
	---------------------
	Output (list) = list of time corrected measured lightcurves.
	---------------------
	"""

	if target == 1:

		print
		print "------ subtracting time delay due to light propagation from asteroid to Earth ------" 

		sec_per_au = 149598000. / 299792.458
		dist = cdr1.load_file(argument)
		cubic_interp = interpolate.interp1d(dist[:,0], dist[:,1], kind='cubic')

		for i,dd in enumerate(alldata):
			dist_Earth = cubic_interp(dd[:,0])
			alldata[i][:,0] = dd[:,0] - dist_Earth * sec_per_au / 86400.

	elif target ==2:

		print
		print "------ converting Julian Dates to Heliocentric Julian Dates ------" 

		#---------------- References ---------------------------------------------------------------------------------------------------------------#
		#	J.Meeus = "Calculs astronomiques a l'usage des amateurs", SAF (1986)																	#
		#	A.U.Landolt, K.L.Blondeau = "The calculation of heliocentric corrections", Pub. Astron. Soc. Pacific., vol 84 (Dec 1972), pp 784-809	#
		#-------------------------------------------------------------------------------------------------------------------------------------------#
		
		for i,dd in enumerate(alldata):
			T = (dd[:,0] - 2415020.)/36525.
			L = 279.69668 + 36000.76892*T + 0.0003025*T*T
			M = 358.47583 + 35999.04975*T - 0.000150*T*T - 0.0000033*T*T*T
			e = 0.01675104 - 0.0000418*T - 0.000000126*T*T
			C = (1.919460- 0.004789*T - 0.000014*T*T)*np.sin(M*np.pi/180.) + (0.020094 - 0.000100*T)*np.sin(2.*M*np.pi/180.) + 0.000293*np.sin(3.*M*np.pi/180.)
			Lv = L+C
			v = M+C
			R = 1.0000002*(1.-e*e)/(1.+e*np.cos(v*np.pi/180.))
			epsilon = 23.452294- 0.0130125*T - 0.00000164*T*T + 0.000000503*T*T*T

			correction = R*np.cos(Lv*np.pi/180.)*np.cos(argument[0]*np.pi/180.)*np.cos(argument[1]*np.pi/180.) \
						+ R*np.sin(Lv*np.pi/180.)*(np.sin(epsilon*np.pi/180.)*np.sin(argument[1]*np.pi/180.) \
									+np.cos(epsilon*np.pi/180.)*np.cos(argument[1]*np.pi/180.)*np.sin(argument[0]*np.pi/180.))
			
			alldata[i][:,0] = dd[:,0] - 0.0057755 * correction
		
	return alldata

