# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize


def fit_CDR(alldata, period, T0, nb_harmo = 6, show_results = True, show_plot = True):
	"""
	---------------------
	Purpose
	Performs a least square fit of an asteroid's or variable star lightcurve with Fourier series.
	Returns the period, period uncertainty, magnitude amplitude, Julian date of maximum and minimum magnitudes.
	Also returns all the fit parameters and their covariance matrix.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays:
	- each Numpy array in the list has 3 columns,
	- the first column contains Julian dates of the observations, with all time corrections been made
	(the Julian date must be mid-exposure times, and the time correction for light propagation delays must have been made with cdr.time_correction() before calling the function),
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value,
	- one array can contain several nights of observation, but must correspond to a single set of comparison stars.
	* period (float) = first guess of the period, used as the starting point for the least square fit
	* T0 (float) = Reference Julian date. This quantity is subtracted from all the Julian dates used in the least square fit calculation, to avoid large numbers.
	The returned Julian dates for min and max magnitudes are the first min and max dates after this reference Julian date.
	* nb_harmo (integer) = optional parameter, default value is 6. Number of Fourier harmonics used for the fit. If F is the fundamental frequency, then the highest frequency used for the fit is nb_harmo*F (6*F by default).
	* show_results (boolean) = if True, then all output parameters are displayed in the Python console. If False, nothing is displayed.
	* show_plot (boolean) = if True, a plot showing the magnitude as a function of phase is displayed. It shows all the measured magnitudes, and the least square fit curve.
	With this plot the user can check visually that the Fourier fit was made correctly, with an appropriate number of harmonics, and a correct starting period.
	---------------------
	Outputs = popt, pcov, period, dperiod, amplitude, maxdate, mindate
	* popt (Numpy array) = this array contains all the fit parameters.
	If the input list of magnitudes (alldata) contains P series of measurements with different magnitude offsets, and if nb_harmo = N then the Numpy array has (1+2*N+P) elements.
	- popt[0] is the fundamental period
	- popt[1:1+N] are the amplitudes of the cosine terms with frequency i*F = i/popt[0] with i=1,2,..,N
	- popt[1+N:1+2*N] are the amplitudes of the sine terms with frequency i*F = i/popt[0] with i=1,2,..,N
	- popt[1+2*N:1+2*N+P] are the magnitude offsets to be used for each serie of measured magnitudes in the alldata input
	* pcov (Numpy array) = this array is a square matrix of size (1+2*N+P)*(1+2*N+P) equal to the covariance matrix obtained with the least square fit.
	The rms uncertainty of the fit parameter popt[k] is equal to np.sqrt(pcov[k,k]).
	* period (float) = fundamental period of the lightcurve in days.
	* dperiod (float) = rms uncertainty of period.
	* amplitude (float) = peak to peak amplitude of the lightcurve.
	* maxdate (float) = Julian date of the lightcurve maximum magnitude (weakest brightness).
	* mindate (float) = Julian date of the lightcurve minimum magnitude (strongest brightness).
	---------------------
	"""

	print
	print "------ least square fit with Fourier series ------" 

	#fit parameters
	offsets = np.zeros((len(alldata)))
	an = np.ones((nb_harmo))
	bn = np.ones((nb_harmo))
	p = [period] + list(an) + list(bn) + list(offsets)
		
	#fit function
	def CDR(alldata, *p):
		#global T0
		result = np.array([])
		for i,dd in enumerate(alldata):
			arg = (dd[:,0]-T0)*2.*np.pi/p[0]
			fit = p[i+1+2*nb_harmo]
			for j in range(nb_harmo):
				fit = fit + p[j+1]*np.cos(np.double(j+1)*arg) + p[j+1+nb_harmo]*np.sin(np.double(j+1)*arg)
			result = np.hstack((result,np.array(fit)))
		return result
		
	#re-organizing measurement data
	ydata = np.zeros((0.))
	sigmadata = np.zeros((0.))

	for i,dd in enumerate(alldata):
		ydata = np.hstack((ydata,dd[:,1]))
		sigmadata = np.hstack((sigmadata,dd[:,2]))

	#fitting
	popt, pcov = optimize.curve_fit(CDR, alldata, ydata, p0=p, sigma = sigmadata)

	#calculating period and period uncertainty
	period = popt[0]
	dperiod = np.sqrt(pcov[0,0])
	
	#calculating amplitude
	x = np.linspace(0.,1.,10000)
	y = 0.
	for j in range(nb_harmo):
		y = y + popt[j+1]*np.cos(np.double(j+1)*x*2.*np.pi) + popt[j+1+nb_harmo]*np.sin(np.double(j+1)*x*2.*np.pi)
	amplitude = np.max(y) - np.min(y)

	#instants of min and max
	minphase = x[np.where(y==np.min(y))]
	maxphase = x[np.where(y==np.max(y))]
	mindate = T0 + popt[0]*minphase
	maxdate = T0 + popt[0]*maxphase
		
	if show_results:
		print
		print "------ fit results ------"	
		print
		print "fit result=",popt
		print "covariance matrix=",pcov
		print
		print "period (days)=",popt[0],"+/-",dperiod
		print "amplitude (mag)=",amplitude
		print "date of max magnitude=",maxdate
		print "date of min magnitude=",mindate

	if show_plot:
		plt.figure()
		for i,dd in enumerate(alldata):
			plt.plot((dd[:,0]-T0)/popt[0]-np.floor((dd[:,0]-T0)/popt[0]), dd[:,1]-popt[i+1+2*nb_harmo],'+')
		plt.plot(x,y,'b')
		plt.xlim([0.,1.])
		plt.grid()
		plt.xlabel('Phase')
		plt.ylabel('Magnitude')

	return popt, pcov, period, dperiod, amplitude, maxdate, mindate
