# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt
from scipy import optimize

def fit_givenperiod(alldata, period, T0, nb_harmo = 6):
	"""
	---------------------
	Purpose
	Performs a least square fit of an asteroid's or variable star lightcurve with Fourier series for a user-specified period, and returns the corresponding rms magnitude error.
	This function can be used to plot a curve of rms magnitude error as a function of period, and to search for the most likely period.
	---------------------
	Inputs
	* alldata (list) = list of Numpy arrays:
	- each Numpy array in the list has 3 columns,
	- the first column contains Julian dates of the observations, with all time corrections been made
	(the Julian date must be mid-exposure times, and the time correction for light propagation delays must have been made with cdr.time_correction() before calling the function),
	- the second one contains the differential magnitudes of the target (asteroid or star),
	- the third one contains an estimation of the magnitude noise for each magnitude value (not used in this function),
	- one array can contain several nights of observation, but must correspond to a single set of comparison stars.
	* period (float) = user specified period used for the least square fit. It is a constant parameter and will not be optimised.
	* T0 (float) = Reference Julian date. This quantity is subtracted from all the Julian dates used in the least square fit calculation, to avoid large numbers.
	* nb_harmo (integer) = optional parameter, default value is 6. Number of Fourier harmonics used for the fit. If F is the fundamental frequency, then the highest frequency used for the fit is nb_harmo*F (6*F by default).
	---------------------
	Output (float) = rms magnitude error.
	---------------------
	"""

	print "... least square fit with Fourier series, period = ",period 

	#fit parameters
	offsets = np.zeros((len(alldata)))
	an = np.ones((nb_harmo))
	bn = np.ones((nb_harmo))
	p = list(an) + list(bn) + list(offsets)
		
	#fit function
	def CDR(alldata, *p):
		#global T0
		result = np.array([])
		for i,dd in enumerate(alldata):
			arg = (dd[:,0]-T0)*2.*np.pi/period
			fit = p[i+2*nb_harmo]
			for j in range(nb_harmo):
				fit = fit + p[j]*np.cos(np.double(j+1)*arg) + p[j+nb_harmo]*np.sin(np.double(j+1)*arg)
			result = np.hstack((result,np.array(fit)))
		return result
		
	#re-organizing measurement data
	ydata = np.zeros((0.))
	for i,dd in enumerate(alldata):
		ydata = np.hstack((ydata,dd[:,1]))

	#fitting
	popt, pcov = optimize.curve_fit(CDR, alldata, ydata, p0=p)

	#calculating squared error
	return np.sum(((ydata - CDR(alldata, *popt))**2.).flatten())
