# -*- coding: iso_8859_1 -*-

import numpy as np
import matplotlib.pyplot as plt

def compute_statistics(mag, nbref = 20, plotting = False):

	mag2 = np.ma.masked_array(mag,np.isnan(mag))

	#-----------------------------------------------------------------------------------------------------------------------
	#		1- Computing statistics for raw magnitudes
	#-----------------------------------------------------------------------------------------------------------------------

	#--- filtering objects with >20% of wrong measurements (stars at edge, close to saturation, or wrong detections) ---
	test_star = np.sum(mag2.mask,axis=0)
	total = np.shape(mag2)[0]

	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		if test_star[i] >= 0.2*total:
			mag2.mask[:,i]=True		
			test_star[i] = total
		
	print str(np.sum(test_star >= 0.2*total)) + " objects have more than 20% of wrong measurements, and have been removed"

	#--- computing statistics for remaining objects ---
	signal = np.mean(mag2,axis=0)
	noise_global = np.std(mag2, ddof=0, axis=0)

	noise_local = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		ind = ~(mag2.mask[:,i])	#indexing unmasked measurements
		if np.sum(ind)==0:
			noise_local.append(np.nan)
		else:
			mag3 = mag2[:,i][ind]
			x = np.std(mag3[2:-2] - (mag3[0:-4]+mag3[1:-3]+mag3[2:-2]+mag3[3:-1]+mag3[4:])/5.,ddof=0, axis=0) / np.sqrt(1.-1./5.)
			noise_local.append(x)

	noise_local = np.array(noise_local)

	#-----------------------------------------------------------------------------------------------------------------------
	#		2- Selecting reference stars
	#-----------------------------------------------------------------------------------------------------------------------

	#--- finding candidates for reference stars ---
	candid_ref = []
	noise_ref = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		if test_star[i] == 0.:
			candid_ref.append(i)
			noise_ref.append(noise_local[i])

	noise_ref = np.array(noise_ref)
	candid_ref = np.array(candid_ref)

	print
	print "nb of reference stars found:", len(candid_ref)

	ind = list(np.argsort(noise_ref))
	indices = candid_ref[ind]
	#print noise_ref[ind[:50]]
	#print candid_ref[ind[:50]]

	#--- refining selection of reference stars ---
	indices = list(indices[:2*nbref])
	
	for i in range(nbref):
		refmagtemp = np.mean(mag2[:,indices],axis=1)
		diffrefmagtemp = mag2[:,indices] - np.tile(refmagtemp,(len(indices),1)).transpose()
		amplitude = np.max(diffrefmagtemp,axis=0) - np.min(diffrefmagtemp,axis=0)
		ind = list(np.argsort(amplitude))
		indices.remove(indices[ind[-1]])

	#-----------------------------------------------------------------------------------------------------------------------
	#		3- Computing statistics for differential magnitudes
	#-----------------------------------------------------------------------------------------------------------------------

	#--- calculates average magnitude of nbref=20 best reference stars ---
	refmag = np.mean(mag2[:,indices],axis=1)

	#--- calculates differential magnitudes ---
	refmag = np.tile(refmag,(np.shape(mag2)[1],1)).transpose()
	diff_mag = mag2 - refmag

	#--- computing statistics again ---
	diff_signal = np.mean(diff_mag,axis=0)
	diff_noise_global = np.std(diff_mag, ddof=0, axis=0)

	diff_noise_local = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		ind = ~(mag2.mask[:,i])	#indexing unmasked measurements
		if np.sum(ind)==0:
			diff_noise_local.append(np.nan)
		else:
			diff_mag2 = diff_mag[:,i][ind]
			x = np.std(diff_mag2[2:-2] - (diff_mag2[0:-4]+diff_mag2[1:-3]+diff_mag2[2:-2]+diff_mag2[3:-1]+diff_mag2[4:])/5.,ddof=0, axis=0) / np.sqrt(1.-1./5.)
			diff_noise_local.append(x)

	diff_noise_local = np.array(diff_noise_local)

	#-----------------------------------------------------------------------------------------------------------------------
	#		4- Flagging quality of measurements
	#-----------------------------------------------------------------------------------------------------------------------
	
	quality_flag = []
	for i in range(np.shape(mag2)[1]):		#loop on all measured objects
		ind = ~(diff_mag.mask[:,i])
		if test_star[i]==total:
			quality_flag.append(-1)
		else:
			c4 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.50,axis=0) >  0)
			c3 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.25,axis=0) >  0)
			c2 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.10,axis=0) >  0)
			c1 = (np.sum(np.abs(diff_mag[ind,i][1:]-diff_mag[ind,i][:-1])>0.05,axis=0) >  0)
			
			quality_flag.append(c4*4 + c3*np.invert(c4)*3 + c2*np.invert(c3)*np.invert(c4)*2 + c1*np.invert(c2)*np.invert(c3)*np.invert(c4)*1)

	#0 --> measurement is fine
	#1 --> measurement is slightly degraded but still fine
	#2 --> measurement is a bit more degraded
	#3 --> measurement has average quality
	#4 --> measurement has poor quality
	#-1 --> measurement has not been accepted because quality is much too low
	
	#-----------------------------------------------------------------------------------------------------------------------
	#		5- Formatting results
	#-----------------------------------------------------------------------------------------------------------------------

	if plotting:
		plt.ion()
		plt.figure()
		plt.scatter(diff_signal,diff_noise_global)
		plt.grid()
		plt.xlabel('Instrumental magnitude')
		plt.ylabel('Magnitude noise')
		plt.draw()

	stats_raw = [signal, noise_global, noise_local]
	stats_diff = [diff_signal, diff_noise_global, diff_noise_local]
	
	return diff_mag, stats_diff, quality_flag, candid_ref[indices[:nbref]]
